package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityGiveRankTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityGiveTargetTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.activity.RankTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.service.activity.ActivityFestivalService;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.service.user.UserWatchRelationService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.*;
import com.starbuds.server.service.task.entity.activity.AutoGiveParamVO;
import com.starbuds.server.service.task.util.SpringContextUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.*;

/**
 * 奖励自动发放
 *
 * @Author: lujiao
 * @Date: 2021/9/23 17:31
 */
@Component
@Slf4j
public class ActivityAutoGiveRewardJob {

    @Resource
    ActivityGiveRewardConfigDAO rewardConfigDAO;

    @Resource
    ActivityGiveConfigDAO configDAO;

    @Resource
    ActivityGiveRewardConfigItemDAO itemDAO;

    @Resource
    ActivityGiveRewardBatchHisDAO batchHisDAO;

    @Resource
    ActivityGiveRewardBatchDetailHisDAO batchDetailHisDAO;

    @Reference(version = "1.0.0")
    ActivityService activityService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    UserWatchRelationService userWatchRelationService;

    @Reference(version = "1.0.0")
    ActivityFestivalService activityFestivalService;

    @Reference(version = "1.0.0")
    ImService imService;

    @XxlJob(value = JobHandlers.ACTIVITY_AUTO_GIVE_REWARD)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> activityAutoGiveRewardHandler(String param) {
        // 首先获取配置列表
        // 根据类型获取榜单数据
        // 获取榜单数据以后进行发放
        // 添加发放记录
        log.info("开始自动发放任务");
        AutoGiveParamVO vo = JSON.parseObject(param, AutoGiveParamVO.class);
        List<ActivityGiveConfigDO> configList = configDAO.getUseConfig();
        if (CollectionUtil.isNotEmpty(configList)) {
            configList.forEach(i -> {
                giveRewardByConfig(i, vo);
            });
        }
        return ReturnT.SUCCESS;
    }

    /**
     * 进行判断, 这里不能使用事务, 因为可能造成礼物发放了, 但是插入记录回滚, 造成重复发放
     *
     * @param config
     */
    public void giveRewardByConfig(ActivityGiveConfigDO config, AutoGiveParamVO vo) {
        List<ActivityGiveRewardConfigDO> rewardConfigList = rewardConfigDAO.getUseRewardConfig(config.getConfigId());
        ActivityDO activityDo = activityService.getActivity(config.getActivityId());
        Long yesterday = DateUtil.getDayBegin() - DateUtil.DAY_OF_MILLION;
        Integer yesterdayInt = DateUtil.getIntDate(new Date(yesterday));
        if (CollectionUtil.isNotEmpty(rewardConfigList)) {
            Integer dateInt = DateUtil.getIntDate();
            for (ActivityGiveRewardConfigDO item : rewardConfigList) {
                if (item.getRankType() == ActivityGiveRankTypeEnum.DAY.getValue()) {
                    int begin = DateUtil.getIntDate(new Date(activityDo.getActivityBeginTime()));
                    int end = DateUtil.getIntDate(new Date(activityDo.getActivityEndTime()));
                    if (!(begin <= yesterdayInt && end >= yesterdayInt)) {
                        continue;
                    }
                } else if (item.getRankType() == ActivityGiveRankTypeEnum.WEEK.getValue()) {
                    // 只有周一可以
                    if (LocalDate.now().getDayOfWeek().getValue() != 1) {
                        continue;
                    }
                } else if (item.getRankType() == ActivityGiveRankTypeEnum.TOTAL.getValue()) {
                    // 只有在结束日第二天发放
                    if (!DateUtil.getIntDate(new Date(activityDo.getActivityEndTime())).equals(yesterdayInt)) {
                        continue;
                    }
                }
                log.info("开始" + config.getConfigRewardName() + "发放任务");
                List<ActivityGiveRewardConfigItemDO> itemDoList = itemDAO.getItemWards(item.getRewardConfigId());
                Map<Integer, List<ActivityGiveRewardConfigItemDO>> rewardMap = mergeRewardByRank(itemDoList);
                List<Long> userIds = getActivityRankIds(item, activityDo, vo);
                log.info("发放用户列表 : " + userIds.toString());
                if (CollectionUtil.isNotEmpty(userIds)) {
                    Long batchId = identityService.getId();
                    Long rewardCost = 0L;
                    String remarks = String.format("活动%s榜单奖励赠送", config.getConfigRewardName());
                    for (int i = 0; i < userIds.size(); i++) {
                        Long userId = userIds.get(i);
                        log.info("用户 :" + userId + "名次 :" + (i + 1) + "开始发放");
                        List<ActivityGiveRewardConfigItemDO> rewards = rewardMap.get(i + 1);
                        log.info("礼物列表 : " + JSON.toJSONString(rewards, true));
                        if (CollectionUtil.isNotEmpty(rewards)) {
                            // 判断之前是否已经发放了奖励
                            for (ActivityGiveRewardConfigItemDO reward : rewards) {
                                log.info("用户ID : {} || 礼物ID {} 开始检查发放", userId, reward.getGiftId());
                                UserDO userDO = userService.getUserById(userId);
                                if (userDO == null) {
                                    continue;
                                }
                                if (batchDetailHisDAO.checkExist(userDO.getUserNo(), activityDo.getActivityId(), item.getRankType(), reward.getGiftId(), dateInt) == 0) {
                                    Long orderId = identityService.getId();
                                    bagService.addItem(userId, reward.getGiftId(), reward.getGiftType(), reward.getQuantity(), reward.getExpireTime(), remarks, orderId);

                                    // 插入日志
                                    ActivityGiveRewardBatchDetailHisDO hisDetailDO = new ActivityGiveRewardBatchDetailHisDO();
                                    hisDetailDO.setHisId(orderId);
                                    hisDetailDO.setBatchId(batchId);
                                    hisDetailDO.setUserId(userDO.getUserNo());
                                    hisDetailDO.setUserName(userService.getUserName(userId));
                                    hisDetailDO.setRank(i + 1);
                                    hisDetailDO.setGiftId(reward.getGiftId());
                                    hisDetailDO.setGiftName(reward.getGiftName());
                                    hisDetailDO.setIcon(reward.getIcon());
                                    hisDetailDO.setQuantity(reward.getQuantity());
                                    hisDetailDO.setCost(reward.getCost());
                                    hisDetailDO.setCreateTime(System.currentTimeMillis());
                                    hisDetailDO.setCreateDate(dateInt);
                                    hisDetailDO.setActivityId(config.getActivityId());
                                    hisDetailDO.setRankType(item.getRankType());
                                    Long cost = 0L;
                                    if (reward.getCost() != null) {
                                        cost = reward.getCost();
                                    }
                                    rewardCost = cost * reward.getQuantity() + rewardCost;
                                    batchDetailHisDAO.insert(hisDetailDO);
                                    handlerMsgContent(config, activityDo, reward, userId);
                                }
                            }
                        }
                    }
                    ActivityGiveRewardBatchHisDO hisDO = new ActivityGiveRewardBatchHisDO();
                    hisDO.setBatchId(batchId);
                    hisDO.setRewardConfigId(item.getRewardConfigId());
                    hisDO.setConfigId(item.getConfigId());
                    hisDO.setActivityId(activityDo.getActivityId());
                    hisDO.setActivityName(activityDo.getActivityName());
                    hisDO.setConfigRewardName(config.getConfigRewardName());
                    hisDO.setTargetType(item.getTargetType());
                    hisDO.setRankType(item.getRankType());
                    hisDO.setRewardValue(rewardCost);
                    hisDO.setCreateDate(dateInt);
                    hisDO.setCreateTime(System.currentTimeMillis());
                    batchHisDAO.insert(hisDO);
                }
                log.info("结束" + config.getConfigRewardName() + "发放任务");
            }
        }
    }

    private Map<Integer, List<ActivityGiveRewardConfigItemDO>> mergeRewardByRank(List<ActivityGiveRewardConfigItemDO> itemDoList) {
        Map<Integer, List<ActivityGiveRewardConfigItemDO>> res = new HashMap<>();
        if (CollectionUtil.isNotEmpty(itemDoList)) {
            itemDoList.forEach(i -> {
                List<ActivityGiveRewardConfigItemDO> tmpList = res.get(i.getRank());
                if (tmpList == null) {
                    tmpList = new ArrayList<>();
                }
                tmpList.add(i);
                res.put(i.getRank(), tmpList);
            });
        }
        return res;
    }

    private void handlerMsgContent(ActivityGiveConfigDO configDO, ActivityDO activityDo, ActivityGiveRewardConfigItemDO reward, Long userId) {
        String content = configDO.getNotifyContent();
        content.replaceAll("#活动名称#", activityDo.getActivityName());
        content.replaceAll("#奖励名称#", reward.getGiftName());
        content.replaceAll("#数量#", reward.getQuantity() + "");

        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle("奖励通知");
        String body = String.format(content);
        systemMsg.setBody(body);
        systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
        imService.sendCustomImMsg(null, userId, systemMsg, null);
    }

    private List<Long> getActivityRankIds(ActivityGiveRewardConfigDO item, ActivityDO activityDo, AutoGiveParamVO vo) {
        List<Long> res = new ArrayList<>();
        Long startTime = 0L;
        Long endTime = 0L;
        if (item.getRankType() == ActivityGiveRankTypeEnum.DAY.getValue()) {
            startTime = DateUtil.getDayBegin() - DateUtil.DAY_OF_MILLION;
            endTime = DateUtil.getDayBegin() - 1;
        } else if (item.getRankType() == ActivityGiveRankTypeEnum.WEEK.getValue()) {
            startTime = DateUtil.getWeekStart() - DateUtil.WEEK_OF_MILLION;
            endTime = DateUtil.getWeekStart() - 1;
            if (startTime < activityDo.getActivityBeginTime()) {
                startTime = activityDo.getActivityBeginTime();
            }
            if (endTime > activityDo.getActivityEndTime()) {
                endTime = activityDo.getActivityEndTime();
            }
        } else if (item.getRankType() == ActivityGiveRankTypeEnum.TOTAL.getValue()) {
            startTime = activityDo.getActivityBeginTime();
            endTime = activityDo.getActivityEndTime();
        }
        // todo 根据不同类型选择不同用户
        if (item.getTargetType() == ActivityGiveTargetTypeEnum.SEND_GIFT_RANK.getValue()) {
            // todo 暂不支持
        } else if (item.getTargetType() == ActivityGiveTargetTypeEnum.RECEIVE_GIFT_RANK.getValue()) {
            // todo 暂不支持
        } else if (item.getTargetType() == ActivityGiveTargetTypeEnum.SPACE_TRAVEL.getValue()) {
            List<ActivityStatRankDO> tmpList = activityFestivalService.getOctoberRank(vo.getLotteryActivityId(), startTime, endTime);
            if (CollectionUtil.isNotEmpty(tmpList)) {
                for (ActivityStatRankDO hisDO : tmpList) {
                    res.add(hisDO.getUserId());
                }
            }
        } else if (item.getTargetType() == ActivityGiveTargetTypeEnum.WATCH_RANK.getValue()) {
            res = userWatchRelationService.getDurationRank(startTime, endTime, item.getLimitRank());
        }
        return res;
    }


}
