package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.statistics.StatCommonTypeEnum;
import com.starbuds.server.common.pojo.data.BagItem;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.PropsDO;
import com.starbuds.server.common.pojo.db.StatCommonDO;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.GiftDAO;
import com.starbuds.server.service.task.dao.PropsDAO;
import com.starbuds.server.service.task.dao.StatCommonDAO;
import com.starbuds.server.service.task.dao.UserBagDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/6/2 15:38
 * @Description 用户背包
 */
@Slf4j
@Component
public class BagGiftJob {

    @Resource
    UserBagDAO userBagDAO;

    @Resource
    GiftDAO giftDAO;

    @Resource
    StatCommonDAO statCommonDAO;

    @Resource
    PropsDAO propsDAO;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.BAG_GIFT)
    public ReturnT<String> bagGiftHandler(String param) {
        log.info("执行用户背包存量礼物价值 {}", param);

        long curTime = System.currentTimeMillis();

        String yyyyMMdd = DateUtil.format(new Date(), "yyyyMMdd");

        // 存量礼物
        List<BagItem> giftItems = userBagDAO.selectUserBag(BagItemTypeEnum.GIFT.getValue(), curTime, null);

        log.info("礼物数量{}", giftItems.size());

        if (!CollectionUtils.isEmpty(giftItems)) {
            // 渠道id
            Set<Long> channelIds = giftItems.stream().map(BagItem::getChannelId).collect(Collectors.toSet());
            List<StatCommonDO> list = new ArrayList<>();
            for (Long channelId : channelIds) {
                Map<Long, Long> giftQuantityMap = giftItems.stream().filter(e -> e.getChannelId().equals(channelId)).collect(Collectors.groupingBy(BagItem::getItemId, Collectors.summingLong(BagItem::getQuantity)));
                Set<Long> giftIdSet = giftQuantityMap.keySet();
                // 礼物id => 礼物价格
                Map<Long, GiftDO> giftPriceMap = giftDAO.selectPrices(new ArrayList<>(giftIdSet));
                // 礼物id => 礼物价值
                Map<Long, Long> giftRewardMap = new HashMap<>();
                for (Long giftId : giftIdSet) {
                    Long giftPrice = giftPriceMap.get(giftId).getGiftPrice();
                    Long giftQuantity = giftQuantityMap.get(giftId);
                    giftRewardMap.put(giftId, giftQuantity * giftPrice);
                }

                // 总价值
                BigDecimal total = BigDecimal.ZERO;
                for (Map.Entry<Long, Long> entry : giftRewardMap.entrySet()) {
                    total = total.add(Convert.toBigDecimal(entry.getValue()));
                }

                StatCommonDO bagDO = new StatCommonDO();
                bagDO.setStatType(StatCommonTypeEnum.BAG_GIFT_STOCK_VALUE.getValue());
                bagDO.setChannelId(channelId);
                bagDO.setStatValue(total);
                bagDO.setStatDate(Convert.toLong(yyyyMMdd));
                bagDO.setCreateTime(curTime);
                list.add(bagDO);
            }

            statCommonDAO.batchInsert(list);
            log.info("存量礼物保存成功");
        }

        // 碎片道具
        List<BagItem> propsItems = userBagDAO.selectUserBag(BagItemTypeEnum.PROPS.getValue(), curTime, PropsTypeEnum.GIFT_FRAGMENT.getValue());
        log.info("碎片道具数量{}", propsItems.size());

        if (!CollectionUtils.isEmpty(propsItems)) {
            // 渠道id
            Set<Long> channelIds = propsItems.stream().map(BagItem::getChannelId).collect(Collectors.toSet());
            List<StatCommonDO> list = new ArrayList<>();
            for (Long channelId : channelIds) {
                // idemId => quantity
                Map<Long, Long> itemQuantityMap = propsItems.stream().filter(e -> e.getChannelId().equals(channelId)).collect(Collectors.groupingBy(BagItem::getItemId, Collectors.summingLong(BagItem::getQuantity)));
                Set<Long> propsIdSet = itemQuantityMap.keySet();
                // 道具
                Map<Long, PropsDO> propsMap = propsDAO.selectPropsMap(new ArrayList<>(propsIdSet));
                // 礼物id
                List<Long> giftIds = propsItems.stream().map(e -> Convert.toLong(e.getPropsExtString())).collect(Collectors.toList());
                // 礼物id => 礼物价格
                Map<Long, GiftDO> giftPriceMap = giftDAO.selectPrices(giftIds);
                // 礼物id => 礼物价值
                Map<Long, Long> fragmentRewardMap = new HashMap<>();
                for (Long propsId : propsIdSet) {
                    // 礼物id
                    Long refGiftId = Convert.toLong(propsMap.get(propsId).getPropsExtString());
                    // 礼物数量
                    Long propsQuantity = itemQuantityMap.get(propsId);
                    // 礼物价格
                    Long giftPrice = giftPriceMap.get(refGiftId).getGiftPrice();
                    // 合成所需数量
                    Integer extInt = propsMap.get(propsId).getPropsExtInt();
                    // 道具碎片单价 = 礼物价格/合成所需数量
                    if (extInt == null) {
                        continue;
                    }
                    long propsFragmentPrice = giftPrice / Convert.toLong(extInt);
                    fragmentRewardMap.put(propsId, propsQuantity * propsFragmentPrice);
                }

                // 总价值
                BigDecimal total = Convert.toBigDecimal(fragmentRewardMap.values().stream().mapToDouble(e -> Convert.toDouble(e)).sum());
                StatCommonDO bagDO = new StatCommonDO();
                bagDO.setStatType(StatCommonTypeEnum.FRAGMENT_PROPS_STOCK_VALUE.getValue());
                bagDO.setChannelId(channelId);
                bagDO.setStatValue(total);
                bagDO.setStatDate(Convert.toLong(yyyyMMdd));
                bagDO.setCreateTime(curTime);
                list.add(bagDO);
            }

            statCommonDAO.batchInsert(list);
            log.info("存量道具碎片保存成功");
        }


        log.info("背包存量定时任务执行成功");
        return ReturnT.SUCCESS;
    }

}
