package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.activity.ActivityHeadlineLotteryPeriodStatusEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.db.ActivityDO;
import com.starbuds.server.common.pojo.db.ActivityHeadlineConfigDO;
import com.starbuds.server.common.pojo.db.ActivityHeadlinePeriodDO;
import com.starbuds.server.common.pojo.message.im.activity.HeadlineLotteryPeriodStatusMsg;
import com.starbuds.server.common.pojo.mq.QueueHeadlineLotteryPeriodOverMsg;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.activity.HeadlineLotteryActivityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ActivityHeadlinePeriodDAO;
import com.starbuds.server.service.task.rocket.RocketProducer;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.concurrent.TimeUnit;

/**
 * 头条抽奖活动任务
 *
 * @author Lay
 * @date 2020/12/28
 */
@Component
@Slf4j
public class HeadlineLotteryActivityJob {

    @Reference(version = "1.0.0")
    ActivityService activityService;

    @Reference(version = "1.0.0")
    HeadlineLotteryActivityService headlineLotteryActivityService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Resource
    ActivityHeadlinePeriodDAO activityHeadlinePeriodDAO;

    @Resource
    RocketProducer rocketProducer;

    @XxlJob(value = JobHandlers.HEADLINE_LOTTERY_PERIOD_CREATE)
    public ReturnT<String> headlineLotteryPeriodCreateHandler(String param) {
        ActivityHeadlineConfigDO config;
        Calendar calendar = Calendar.getInstance();
        // DateUtil 会影响 calendar 对象本身
        Calendar calendarSearch = Calendar.getInstance();
        calendarSearch.setTimeInMillis(calendar.getTimeInMillis());

        if (StringUtils.isNotBlank(param)) {
            config = headlineLotteryActivityService.getConfigById(Convert.toLong(param));
        } else {
            String hour = String.valueOf(calendar.get(Calendar.HOUR_OF_DAY));
            // 查询当前小时任务
            config = headlineLotteryActivityService.getConfigByTimeExp(hour);
        }

        if (config != null) {
            ActivityDO activity = activityService.getActivity(config.getActivityId());
            if (activity == null) {
                log.warn("抢头条活动已被停用");
            }

            // 查询最近期号
            Integer latestPeriodId = activityHeadlinePeriodDAO.selectLastPeriodId(config.getActivityId(), DateUtil.beginOfDay(calendarSearch).getTimeInMillis(), DateUtil.endOfDay(calendarSearch).getTimeInMillis());
            int periodId;
            if (latestPeriodId == null) {
                SimpleDateFormat sdf = new SimpleDateFormat("MMdd");
                periodId = Convert.toInt(String.valueOf(calendar.get(Calendar.YEAR)).substring(2) + sdf.format(calendar.getTime()) + "001");
            } else {
                periodId = latestPeriodId + 1;
            }

            if (redisService.setIfAbsent(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_GOING, String.format("%d:%d", config.getConfigId(), periodId), config.getTimeDuration(), TimeUnit.MILLISECONDS)) {
                try {
                    // 创建期数
                    ActivityHeadlinePeriodDO period = new ActivityHeadlinePeriodDO();
                    period.setConfigId(config.getConfigId());
                    period.setPeriodId(periodId);
                    period.setPeriodStatus(ActivityHeadlineLotteryPeriodStatusEnum.Going.getValue());
                    period.setUsersCount(0);
                    period.setTimesCount(0);
                    period.setPeriodRewardConfig(config.getRewards());
                    period.setCreateTime(calendar.getTimeInMillis());
                    period.setOverTime(calendar.getTimeInMillis() + config.getTimeDuration());
                    period.setPriceType(config.getPriceType());
                    period.setPrice(config.getPrice());
                    activityHeadlinePeriodDAO.insert(period);

                    // 创建结束消息
                    QueueHeadlineLotteryPeriodOverMsg overMsg = new QueueHeadlineLotteryPeriodOverMsg();
                    overMsg.setConfigId(config.getConfigId());
                    overMsg.setPeriodId(periodId);
                    rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.HEADLINE_LOTTERY_PERIOD_OVER, overMsg.serialize(), period.getOverTime());

                    // 重置房间热度
                    headlineLotteryActivityService.resetRoomHeat();

                    // 广播通知
                    HeadlineLotteryPeriodStatusMsg periodStatusMsg = new HeadlineLotteryPeriodStatusMsg();
                    periodStatusMsg.setActivityName(activity.getActivityName());
                    periodStatusMsg.setPeriodStatus(ActivityHeadlineLotteryPeriodStatusEnum.Going.getValue());
                    periodStatusMsg.setConfigId(config.getConfigId());
                    periodStatusMsg.setPeriodId(periodId);
                    imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, 0L, periodStatusMsg);
                } catch (Exception e) {
                    // 删除 key
                    redisService.delKey(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_GOING);
                    log.error("头条抽奖活动创建失败", e);
                }
            } else {
                log.error("头条活动创建失败：有进行中的活动");
            }
        }

        return ReturnT.SUCCESS;
    }
}
