package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.data.RtcRoomBackground;
import com.starbuds.server.common.pojo.mq.QueueRtcUserExitMsg;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.DataRtcRoomBackgroundDAO;
import com.starbuds.server.service.task.dao.RtcRoomDAO;
import com.starbuds.server.service.task.provider.RongCloudProvider;
import com.starbuds.server.service.task.rocket.RocketProducer;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.ShardingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static com.starbuds.server.service.task.util.SpringContextUtil.isDev;

/**
 * 语聊房在线用户扫描
 *
 * @author Lay
 * @date 2020/8/28
 */
@Component
@Slf4j
public class RoomOnlineScanJob {

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Resource
    DataRtcRoomBackgroundDAO dataRtcRoomBackgroundDAO;

    @Resource
    private RongCloudProvider rongCloudProvider;

    /**
     * 语聊房扫描
     *
     * @return
     */
    @XxlJob(value = JobHandlers.RTC_ROOM_ONLINE_SCAN)
    public ReturnT<String> rtcRoomOnlineScanHandler(String pararms) {
        log.info("触发语聊房在线用户扫描,入参：{}", pararms);
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();

        List<Long> roomIds = new ArrayList<>();
        if (StringUtils.isNotBlank(pararms)) {
            // 处理该房间
            roomIds.add(Convert.toLong(pararms));
        } else {
            roomIds = rtcRoomDAO.selectHeatRoomIds();
        }
        log.info("符合条件的房间数量：{}", roomIds.size());
        if (CollectionUtil.isNotEmpty(roomIds)) {
            for (Long roomId : roomIds) {
                scanOnlineUsers(roomId);
            }
            log.info("已扫描{}个房间", roomIds.size());
            return ReturnT.SUCCESS;
        }
        log.info("暂时无需扫描用户");
        return ReturnT.SUCCESS;
    }

    /**
     * 背景过期时间扫描
     *
     * @param params
     * @return
     */
    @XxlJob(value = JobHandlers.RTC_ROOM_BACKGROUD_SCAN)
    public ReturnT<String> rtcRoomBackgroundScanHandler(String params) {
        List<RtcRoomBackground> scopeBackgroundList = dataRtcRoomBackgroundDAO.selectScopeBackgroud(System.currentTimeMillis());
        List<Integer> backgroudIDList = scopeBackgroundList.stream().map(m -> m.getBackgroundId()).collect(Collectors.toList());
        ;
        if (backgroudIDList.size() > 0) {
            rtcRoomDAO.updateRtcRoomBackground(backgroudIDList);
        }

        return ReturnT.SUCCESS;
    }

    private void scanOnlineUsers(Long roomId) {
//        log.info("开始扫描房间{}的在线用户",roomId);
        // 查询房间在线用户 id
        List<Long> onlineUserIds = rtcUserService.getOnlineUserIds(roomId);
//        log.info("房间：{},在线用户数量：{}", roomId, onlineUserIds.size());
        if (CollectionUtil.isEmpty(onlineUserIds)) {
            return;
        }

        // 判断用户是否在麦位上 在麦位上则忽略
        List<Long> seatUserIds = rtcSeatService.getSeatUserIds(roomId);
        List<Long> finalUserIds = new ArrayList<>();
        for (Long userId : onlineUserIds) {
            if (!seatUserIds.contains(userId)) {
                finalUserIds.add(userId);
            }
        }
        if (CollectionUtil.isEmpty(finalUserIds)) {
            return;
        }

        int count = 1000;
        int totalNum = finalUserIds.size();
        // 查询用户是否在线 如果大于1000个用户要分批处理
        int subNum = getSubNum(totalNum, count);
        for (int i = 0; i < subNum; i++) {
            int pageStart = i * count;
            int pageEnd = (i + 1) * count + 1;
            if (pageEnd > totalNum) {
                pageEnd = totalNum;
            }
            List<Long> subList = finalUserIds.subList(pageStart, pageEnd);

            // 离线用户
            List<Long> offLineUserIds = rongCloudProvider.getImOfflineUserIds(roomId, subList);
            if (CollectionUtil.isEmpty(offLineUserIds)) {
                return;
            }

            if (isDev()) {
                log.info(LogUtil.devLog("获得融云离线用户列表 roomId={} 离线用户数量={} {}"),
                        roomId, offLineUserIds.size(), offLineUserIds);
            } else {
                log.info(LogUtil.devLog("获得融云离线用户列表 roomId={} 离线用户数量={}"),
                        roomId, offLineUserIds.size());
            }

            // 不在线的话丢出离开房间的 MQ 消息
            for (Long userId : offLineUserIds) {
                if (isDev()) {
                    log.info("融云回调用户退出房间, 房间id:{}", roomId);
                }
                QueueRtcUserExitMsg rtcUserExitMsg = new QueueRtcUserExitMsg();
                rtcUserExitMsg.setChannelId(0L);
                rtcUserExitMsg.setExitUserId(userId);
                rtcUserExitMsg.setRoomId(roomId);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_USER_EXIT, rtcUserExitMsg.serialize());
            }
        }
    }

    /**
     * 获取任务次数
     *
     * @param total
     * @param count
     * @return
     */
    private int getSubNum(int total, int count) {
        int subNum = total / count;
        if (total % count > 0) {
            subNum += 1;
        }
        return subNum;
    }
}
