package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ReUtil;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.ScoreRankDO;
import com.starbuds.server.common.pojo.user.ScoreRankTimes;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.user.ScoreRankService;
import com.starbuds.server.service.task.async.ScoreRankPersistenceAsync;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.constant.ScoreRankTypeEnum;
import com.starbuds.server.service.task.dao.ScoreRankDAO;
import com.starbuds.server.service.task.entity.ShardingParam;
import com.starbuds.server.service.task.util.ShardingHelper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.ShardingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 积分排行持久化任务
 *
 * @author Lay
 * @date 2020/2/28
 */
@Component
@Slf4j
public class ScoreRankPersistenceJob {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    ScoreRankService scoreRankService;

    @Resource
    ScoreRankDAO scoreRankDAO;

    @Resource
    ScoreRankPersistenceAsync scoreRankPersistenceAsync;

    /**
     * 积分小时榜处理
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.SCORE_RANK_HOUR)
    public ReturnT<String> hourlyJobHandler(String param) {
        log.info("触发积分小时榜持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 查询小时处理
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHH");
        Integer hour;
        if (StringUtils.isNotBlank(param)) {
            // 有传参取传参的
            hour = Convert.toInt(param);
        } else {
            // 不然就是取上一小时的
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.HOUR_OF_DAY, -1);
            hour = Convert.toInt(sdf.format(calendar.getTime()));
        }

        if (hour != null) {
            // 从缓存中找出所有积分类型的小时榜数据 key
            String pattern = RedisCacheKeys.getScoreRankHourPattern(hour);
            executePersist(ScoreRankTypeEnum.HOURLY, shardingVO, hour, pattern);

            return ReturnT.SUCCESS;
        }

        return ReturnT.FAIL;
    }

    /**
     * 积分日榜处理
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.SCORE_RANK_DAY)
    public ReturnT<String> dailyJobHandler(String param) {
        log.info("触发积分日榜持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 查询日期处理
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer day;
        if (StringUtils.isNotBlank(param)) {
            // 有传参取传参的
            day = Convert.toInt(param);
        } else {
            // 不然就是取前一天的，因为定时任务一定是后一天来统计前一天的
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.DATE, -1);
            day = Convert.toInt(sdf.format(calendar.getTime()));
        }

        if (day != null) {
            // 从缓存中找出所有积分类型的日榜数据 key
            String pattern = RedisCacheKeys.getScoreRankDayPattern(day);
            executePersist(ScoreRankTypeEnum.DAILY, shardingVO, day, pattern);

            return ReturnT.SUCCESS;
        }
        return ReturnT.FAIL;
    }

    /**
     * 积分周榜处理
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.SCORE_RANK_WEEK)
    public ReturnT<String> weeklyJobHandler(String param) {
        log.info("直播周榜持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 周数处理
        Integer weekNum;
        if (StringUtils.isNotBlank(param)) {
            weekNum = Convert.toInt(param);
        } else {
            // 取上一周
            ScoreRankTimes rankTimes = scoreRankService.getRankTimes();
            weekNum = rankTimes.getWeek() - 1;
        }

        if (weekNum != null) {
            // 从缓存中找出所有积分类型的周榜数据 key
            String pattern = RedisCacheKeys.getScoreRankWeekPattern(weekNum);
            executePersist(ScoreRankTypeEnum.WEEKLY, shardingVO, weekNum, pattern);

            return ReturnT.SUCCESS;
        }

        return ReturnT.FAIL;
    }

    /**
     * 积分月榜处理
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.SCORE_RANK_MONTH)
    public ReturnT<String> monthlyJobHandler(String param) {
        log.info("直播月榜持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 月份处理
        Integer monthNum;
        if (StringUtils.isNotBlank(param)) {
            monthNum = Convert.toInt(param);
        } else {
            // 取上个月
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.MONTH, -1);
            monthNum = Convert.toInt(sdf.format(calendar.getTime()));
        }

        if (monthNum != null) {
            // 从缓存中找出所有积分类型的周榜数据 key
            String pattern = RedisCacheKeys.getScoreRankMonthPattern(monthNum);
            executePersist(ScoreRankTypeEnum.MONTHLY, shardingVO, monthNum, pattern);

            return ReturnT.SUCCESS;
        }

        return ReturnT.FAIL;
    }

    /**
     * 执行持久化
     *
     * @param scoreRankType
     * @param shardingVO
     * @param timeNum
     * @param pattern
     */
    private void executePersist(ScoreRankTypeEnum scoreRankType, ShardingUtil.ShardingVO shardingVO, Integer timeNum, String pattern) {
        // 从缓存中找出所有积分类型的日榜数据 key
        List<String> keys = cacheService.getKeys(pattern);
        for (String key : keys) {
            // 根据分片计算处理数据
            Long size = cacheService.zCard(key);
            ShardingParam shardingParam = ShardingHelper.getParam(shardingVO, size);
            if (shardingParam != null) {
                Set<Tuple> tupleSet = cacheService.zReverseRangeWithScores(key, shardingParam.getStart(), shardingParam.getEnd());
                if (tupleSet != null) {
                    List<Tuple> tupleList = new ArrayList<>(tupleSet);
                    tupleList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                    Integer scoreType = Convert.toInt(ReUtil.get(pattern.replace("*", "([0-9]+)"), key, 1));
                    // 执行处理
                    int rankNo = shardingParam.getStart() + 1;
                    Long curTime = System.currentTimeMillis();
                    List<ScoreRankDO> list = new ArrayList<>();
                    for (Iterator<Tuple> iterator = tupleList.iterator(); iterator.hasNext(); ) {
                        Tuple tuple = iterator.next();
                        ScoreRankDO scoreRank = new ScoreRankDO();
                        scoreRank.setTimeNum(timeNum);
                        scoreRank.setScoreType(scoreType);
                        scoreRank.setUserId(Convert.toLong(tuple.getElement()));
                        scoreRank.setScoreValue(Convert.toLong(tuple.getScore()));
                        scoreRank.setRankNo(rankNo);
                        scoreRank.setUpdateTime(curTime);
                        list.add(scoreRank);
                        rankNo++;
                    }

                    scoreRankDAO.batchInsert(scoreRankType.getTable(), list);

                    if (shardingParam.getEnd() == -1) {
                        // 异步检查
                        scoreRankPersistenceAsync.checkSize(30 * 1000L, scoreRankType.getTable(), size, timeNum, scoreType, key);
                    }
                }
            }
        }
    }
}
