package com.starbuds.server.service.task.util;

import lombok.extern.slf4j.Slf4j;
import okhttp3.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/**
 * @Author lujiao
 * @DateTime 2021/7/8 18:07
 * @Description
 */
@Slf4j
public class HttpUtil {

    private static OkHttpClient client;
    private static OkHttpClient clientMaxTimout;

    private static final MediaType MEDIA_TYPE_JSON = MediaType.get("application/json; charset=utf-8");

    static {
        client = new OkHttpClient();
        clientMaxTimout = new OkHttpClient.Builder()
                .callTimeout(30, TimeUnit.SECONDS)
                .readTimeout(60, TimeUnit.SECONDS)
                .build();
    }

    private HttpUtil() {
    }

    public static String post(String url, String json) {
        return post(url, json, null);
    }

    public static String post(String url, String json, Map<String, String> headers) {
        if (headers == null) {
            headers = new HashMap<>();
        }

        RequestBody body = RequestBody.create(json, MEDIA_TYPE_JSON);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .headers(Headers.of(headers))
                .build();
        try (Response response = clientMaxTimout.newCall(request).execute()) {
            if (!response.isSuccessful()) {
                log.warn("Request Failed[{}] {} {}", response.code(), url, json);
            }
            return response.body().string();
        } catch (Exception e) {
            log.error(String.format("Request Failed url = %s body = %s Headers = %s", url, json, headers), e);
        }
        return null;
    }

    public static String postForm(String url, Map<String, String> params) throws IOException {
        Request.Builder builder = new Request.Builder();
        if (params != null) {
            FormBody.Builder body = new FormBody.Builder();
            for (Map.Entry<String, String> param : params.entrySet()) {
                body.add(param.getKey(), param.getValue());
            }
            builder.post(body.build());
        }

        Request request = builder.url(url).build();

        try (Response response = clientMaxTimout.newCall(request).execute()) {
            return response.body().string();
        }
    }

    public static String postForm(String url, Map<String, String> body, Map<String, String> headers) {
        if (headers == null) {
            headers = new HashMap<>();
        }
        FormBody.Builder builder = new FormBody.Builder();
        body.entrySet().stream()
                .filter(Objects::nonNull)
                .forEach(kv -> {
                    builder.add(kv.getKey(), kv.getValue());
                });
        RequestBody formBody = builder.build();
        Request request = new Request.Builder()
                .url(url)
                .post(formBody)
                .headers(Headers.of(headers))
                .build();
        try (Response response = clientMaxTimout.newCall(request).execute()) {
            if (!response.isSuccessful()) {
                log.warn("Request Failed[{}] {} {}", response.code(), url, formBody);
            }
            return response.body().string();
        } catch (Exception e) {
            log.error(String.format("Request Failed url = %s body = %s Headers = %s", url, formBody, headers), e);
            if (e instanceof TimeoutException) {
                return "timeout";
            }
        }
        return null;
    }
}